#ifndef UMBRELLA_TYPES
#define UMBRELLA_TYPES

/* defines the datatype(bitvector) used in security_struct */
#include "umb_bitvector.h"  
#include "umb_hash.h"


#include "umb_signature.h" // Just a test

/* L1 restrictions */
#define SECONDLEVEL    0
#define NOSIGKILL      1
#define NOSIGTERM      2
#define NOSIGQUIT      3
#define NOSIGHUP       4
#define NOSIGTRAP      5
#define NOSIGALRM      6
#define NOSIGCHLD      7
#define NOIP           8
#define NOIRDA         9
#define NOBLUETOOTH   10
#define NOFORK        11

/* other needed definitions */
#define NUMBER_OF_LEVEL1 12
#define MAX_PATH_DEPTH 256 /* how about this depth??? */
#define MAX_FILE_LENGTH 256 /* how about this length??? */
#define MAX_L2 256

/* parsing policy in /proc/umbrella */
#define IS_INT   1
#define IS_SLASH 2
#define IS_COLON 3

/* Just for the sake of readable code ;) */
#define SUCCES 0
#define FAILURE 1
#define ALLOWED 0
#define DENIED 1

/* The fsr struct holds file system restrictions (fsr) from a specific
   process. */
struct fsr {
	char *name;
	struct fsr *succ;
	struct fsr *next;
};

struct fsr *init_fsr(void);
int insert_into_fsr(struct fsr *root, char **path);
int check_restriction_in_fsr(struct fsr *root, char **path);
struct fsr *create_fsr_subtree(char **path);
struct fsr *copy_fsr_tree(struct fsr *source);
void destroy_fsr_tree(struct fsr *target);

/* the security field that lives on all processes */
/* restriction field holds the restrictions that apply to this particular process */
/* child_restriction holds the extra restrictions that should be applied to the next child */  
struct security_struct {
	bitvector level1;
	bitvector child_level1;
	struct fsr *level2;
	char *child_level2[MAX_L2];
	bitvector bin_level1;
	char *bin_level2[MAX_L2];
};

/* free the memory used by a security_struct */
void security_struct_destroy(struct security_struct *target);

/* alloc memory for a security_struct */
struct security_struct *security_struct_create(void);


#endif
