/******************************************************************************/
/* $Header: /cvsroot/umbrella/umbrella-devel/kernel/security/umbrella/umb_hash_l2.c,v 1.13 2004/11/16 18:46:29 ipqw Exp $ */
/******************************************************************************/
/*  AUTHORS:                                                                  */
/******************************************************************************/
/*                                                                            */
/*    Sren Nhr Christensen                                                  */
/*    Kristian Srensen                                                       */
/*    Michel Thryse                                                          */
/*    mailto:umbrella@cs.auc.dk                                               */
/*    http://umbrella.sourceforge.net                                         */
/*                                                                            */
/******************************************************************************/
/*  COPYRIGHT:                                                                */
/******************************************************************************/
/*                                                                            */
/*    Copyright (c) 2004 The Umbrella Security Team.                          */
/*    All rights reserved.                                                    */
/*                                                                            */
/******************************************************************************/
/*  LICENSE:                                                                  */
/******************************************************************************/
/*                                                                            */
/*    This file is part of the Umbrella Security Project and is free          */
/*    software; you can redistribute it and/or modify it under the terms      */
/*    of the GNU Library General Public License as published by the Free      */
/*    Software Foundation;                                                    */
/*    The code is distributed in the hope that it will be useful,             */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/*    Library General Public License for more details.                        */
/*                                                                            */
/*    You should have received a copy of the GNU Library General Public       */
/*    License along with this library; if not, write to the                   */
/*    Free Software Foundation, Inc.,                                         */
/*    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   */
/*                                                                            */
/*    or download a copy from ftp://ftp.gnu.org/pub/gnu/COPYING.LIB-2.0       */
/*                                                                            */
/******************************************************************************/

#include "include/umb_hash.h"
#include "include/umb_hash_l2.h"


/* Inspired from include/linux/dcache.h */
unsigned long kshash(char *string) {
  unsigned long hash=0;
  int length;

  for (length=strlen(string); length; length--) {
          hash = (hash+((*string)<<4)+((*string)>>4))*11;
          string++;
  }

  return (unsigned long) hash;
}


/* Hash table functions for restrictions (hash_structs) */
hashptr hash_table_create() {
	int i = 0;
	
	hashptr table = NULL;
	table = kmalloc(sizeof(struct hash_struct*)*HASH_TABLE_SIZE,GFP_ATOMIC);
	if (table) {
		for (i = 0; i < HASH_TABLE_SIZE; i++)
			table[i] = NULL;
	}
	return table;
}


/* Hash table functions for restrictions (hash_structs) */
void hash_table_destroy(hashptr table) {
	struct hash_struct *current_entry = NULL;
	struct hash_struct *next = NULL;
	int i = 0;

	for (i=0; i<HASH_TABLE_SIZE; i++) {
		current_entry = table[i];
		
		while(current_entry != NULL) {
			next = current_entry->next;
			kfree(current_entry->path);
			kfree(current_entry);
			current_entry = next;
		} 
	}
	kfree(table);
}

void hash_table_copy(hashptr destination, hashptr source) {
	struct hash_struct *current_entry = NULL;	
	int i = 0;
	
	for (i=0; i<HASH_TABLE_SIZE; i++) {
		current_entry = source[i];
		while(current_entry != NULL) {			
			hash_table_insert(destination, current_entry->path);
			current_entry = current_entry->next;
		}
	}
	return;
}
int hash_table_insert(hashptr table, char *string) {
  int mod_hash = kshash(string) % HASH_TABLE_SIZE;
  int i, res = 0, length = strlen(string) + 1;
  char *path;
  struct hash_struct *previous_node, *current_node, *new_node;
  
  /* Look for an existing node in the hashtable */
  /**********************************************/
  current_node = (struct hash_struct *)table[mod_hash];
  
  if (current_node == NULL) {
    /* No existing node found, inserting a new one */
    res = 1;
    
  } else {
    /* Existing node found, finding place for new node */
    
    /* Stepping through all nodes with same mod_hash hash */
    do {
      if(!strncmp(string, current_node->path,
                  strlen(current_node->path)))
        return 0; /* The path is subsumed by another entry */

      /* Stepping to the next loop */
      previous_node = current_node;
      current_node = current_node->next;
      
    } while (current_node != NULL);
  }

  /* Initialization of the the new node (if necessary) */
  /*****************************************************/
  new_node = kmalloc(sizeof(struct hash_struct), GFP_ATOMIC);
  
  if (!new_node)
    return -ENOMEM;
  
  path = kmalloc(sizeof(char)*(length), GFP_ATOMIC);
  
  if (!path) {
    kfree(new_node);
    return -ENOMEM;
  }
  
  for(i=0;i<length;i++)
    path[i] = '\0';
  
  strcpy(path, string);
  new_node->path = path;
  new_node->next = NULL;
  
  /* res = 0 : Existing node found, inserting the new node */
  /* res = 1 : No existing node found, inserting the new node */
  (res ? (table[mod_hash] = new_node) : (previous_node->next = new_node));
  
  return 0;
}

int hash_exists(hashptr table, char *path) {
	unsigned long real_hash = kshash(path);
	int mod_hash = real_hash % HASH_TABLE_SIZE;
	struct hash_struct *current_entry = NULL;
	current_entry = (struct hash_struct *)table[mod_hash];
	
	while(current_entry != NULL) {
		if (!strcmp(path, current_entry->path))
			return 1;
		current_entry = current_entry->next;
	}
	return 0;
}

void hash_table_print(hashptr table) {
	struct hash_struct *current_entry = NULL;	
	int i = 0;
	int j = 0;

	printk("____ hash table print start ____\n");

	for (i=0; i<HASH_TABLE_SIZE; i++) {
		current_entry = table[i];
		/* follow current_entrys */
		j = 0;
		while(current_entry != NULL) {
			printk("HASH: %i  %i  %s\n", i, j, current_entry->path);
			current_entry = current_entry->next;
			j++;
		}
	}
	printk("____ hash table print end ____\n");

	return;
}

