/*****************************************************************************/
/* $Header: /cvsroot/umbrella/umbrella-devel/kernel/security/umbrella/umb_keyring.c,v 1.6 2005/03/16 10:23:24 mthrysoe Exp $ */
/*****************************************************************************/
/*  AUTHOR:                                                                  */
/*****************************************************************************/
/*                                                                           */
/*    Sren Nhr Christensen                                                 */
/*    Kristian Srensen                                                      */
/*    Michel Thryse                                                         */
/*    mailto:umbrella@cs.aau.dk                                              */
/*    http://umbrella.sourceforge.net                                        */
/*                                                                           */
/*****************************************************************************/
/*  COPYRIGHT:                                                               */
/*****************************************************************************/
/*                                                                           */
/*    Copyright (c) 2004 The Umbrella Security Team.                         */
/*    All rights reserved.                                                   */
/*                                                                           */
/*****************************************************************************/
/*  LICENSE:                                                                 */
/*****************************************************************************/
/*                                                                           */
/*    This file is part of the Umbrella Security Project and is free         */
/*    software; you can redistribute it and/or modify it under the terms     */
/*    of the GNU Library General Public License as published by the Free     */
/*    Software Foundation;                                                   */
/*    The code is distributed in the hope that it will be useful,            */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       */
/*    Library General Public License for more details.                       */
/*                                                                           */
/*    You should have received a copy of the GNU Library General Public      */
/*    License along with this library; if not, write to the                  */
/*    Free Software Foundation, Inc.,                                        */
/*    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                  */
/*                                                                           */
/*    or download a copy from ftp://ftp.gnu.org/pub/gnu/COPYING.LIB-2.0      */
/*                                                                           */
/*****************************************************************************/
#include "include/umb_signature.h"
#include "include/umb_keyring.h"
#include <asm/uaccess.h>

struct key *global_keyring[] = {NULL, NULL};

static void read_rawdata(char *file, char *rawdata) {

	int i;
	struct file *f = NULL;
	mm_segment_t orig_fs;

	f = filp_open(file, O_RDONLY, 00);

	if (!f || !f->f_op || !f->f_op->read)
	  printk("WARNING: File (read) object is a null pointer!!!\n");

	f->f_pos = 0;

	orig_fs = get_fs();
	set_fs(KERNEL_DS);	

	i = f->f_op->read(f, rawdata, 370, &f->f_pos);

	/* Release the file object pointer. */
	filp_close(f, 0);
	
	return;
}

static struct key *insert_key_from_disk(char *file) {

	MPI MPI_e, MPI_n  = NULL;
	struct key *key = (struct key *)kmalloc(sizeof(struct key), GFP_ATOMIC);

	int i = 3; /* value?? ask MT */
	int j = 130; /* value?? ask MT */
	
	unsigned char *n =
		(unsigned char *)kmalloc(UMBRELLA_MPI_MAX_SIZE_N, GFP_ATOMIC);
	unsigned char *e =
		(unsigned char *)kmalloc(UMBRELLA_MPI_MAX_SIZE_E, GFP_ATOMIC);

	if(!n || !e) {
		kfree(n);
		kfree(e);
		return NULL;
	}

	read_rawdata(file, key->rawdata);
	
	if (umb_get_pkey(n, e, file) == -1)
		return NULL;
	
	MPI_e = mpi_read_from_buffer(e, &i, 0);
	MPI_n = mpi_read_from_buffer(n, &j, 0);

	key->payload[0] = MPI_n;
	key->payload[1] = MPI_e;
	strcpy(key->id, "umb.pub");  //remember to check to length!
	
	kfree(n);
	kfree(e);
	
	return key;
}

/* Must be rewritten to traverse /etc/umbrella/keys/, and insert all keys */
/* Cannot be called before filesystem is in place */
int init_keyring() {

	char *file1 = "/etc/umbrella/keyring/mt.pub";
	struct key *key = NULL;

	printk(KERN_INFO "Initializing keyring!\n");
	key = insert_key_from_disk(file1);

	if(!key) 
		return -1;
	
	global_keyring[0] = key;
	
	return 0;
}

struct key *get_key(int position) {

	if (!global_keyring[position]) {
		printk(KERN_INFO "No key at pos. %i, trying to initialize keyring\n", position);
		if (init_keyring() == -1) {
			printk(KERN_INFO "Keyring init. failed, check /etc/umbrella/keyring/static_key.pub\n");
			return NULL;
		}
	}
	return global_keyring[position];
}
                                                                                
int write_key_to_disk(struct key *key) {

	struct file *f = NULL;
	char key_path[256];
	mm_segment_t orig_fs;

	strcpy(key_path, "/etc/umbrella/keyring/");
	strcat(key_path, key->id);

	f = filp_open(key_path , O_WRONLY | O_CREAT, 0600);

	if (!f || !f->f_op || !f->f_op->write)
	  printk("WARNING: File (write) object is a null pointer!!!\n");

	f->f_pos = 0;
	orig_fs = get_fs();
	set_fs(KERNEL_DS);	

	f->f_op->write(f, key->rawdata, 370, &f->f_pos);
	filp_close(f, 0);

	return 0;
}
